<?php

namespace EventPortal\Tests\Unit\SignIn;

use Carbon\Carbon;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\SignIn\Models\SignIn;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class SignInTest extends TestCase
{
    use RefreshDatabase;

    /** @test */
    public function is_signed_in_attr_is_corrrect(): void
    {
        $signIn = new SignIn([
            'signed_in_at' => Carbon::now()->subMinutes(5),
        ]);

        $this->assertTrue($signIn->signedIn, 'no sign_out - no signed_out_at check failed');

        $signIn->signed_out_at = Carbon::now()->subMinutes(3);

        $this->assertFalse($signIn->signedIn, 'sign_out more recent - signed_in_at >= signed_out_at check failed');

        $signIn->signed_in_at = Carbon::now();
        $this->assertTrue($signIn->signedIn, 'sign_in more recent - signed_in_at >= signed_out_at check failed');
    }

    /** @test */
    public function signed_in_at_locationable_scope_works(): void
    {
        $eventA = factory(Event::class)->create();
        $eventB = factory(Event::class)->create();
        $eventDelegates = factory(EventDelegate::class, 5)->create();

        collect($eventDelegates)->each(function (EventDelegate $eventDelegate) use ($eventA, $eventB): void {
            $eventDelegate->signInTo($eventA);
            $eventDelegate->signInTo($eventB);

            $signIns = $eventDelegate->signins()->atLocationable($eventA)->get('locationable_id');

            $this->assertEquals(2, $eventDelegate->signins()->count());
            $this->assertEquals(1, $signIns->count());
            $this->assertEquals($eventA->id, $signIns->first()->locationable_id);
        });
    }

    /** @test */
    public function signed_in_for_userable_scope_works(): void
    {
        $event = factory(Event::class)->create();
        $eventDelegates = factory(EventDelegate::class, 5)->create();

        collect($eventDelegates)->each(function (EventDelegate $eventDelegate) use ($event): void {
            $eventDelegate->signInTo($event);
        });

        $eventDelegate = $eventDelegates->first();
        $signIns = $event->signins()->forUserable($eventDelegate)->get('userable_id');

        $this->assertEquals(5, $event->signins()->count());
        $this->assertEquals(1, $signIns->count());
        $this->assertEquals($eventDelegate->id, $signIns->first()->userable_id);
    }

    /** @test */
    public function delegate_can_check_sign_in_status(): void
    {
        $eventDelegate = factory(EventDelegate::class)->create();
        $event = factory(Event::class)->create();

        $this->assertCount(0, $eventDelegate->signins);
        $this->assertFalse($eventDelegate->isSignedInAt($event));

        $signInRecord = SignIn::create([
            'locationable_id' => $event->id,
            'locationable_type' => Event::class,
            'userable_id' => $eventDelegate->id,
            'userable_type' => EventDelegate::class,
            'signed_in_at' => Carbon::now()->subMinutes(5),
        ]);

        $eventDelegate->fresh();

        $this->assertCount(1, $eventDelegate->signIns()->get());
        $this->assertTrue($eventDelegate->isSignedInAt($event));

        $signInRecord->update(['signed_out_at' => Carbon::now()]);

        $eventDelegate->fresh();

        $this->assertEquals(1, $eventDelegate->signins()->count());
        $this->assertFalse($eventDelegate->isSignedInAt($event));
    }

    /** @test */
    public function delegate_can_be_signed_into_an_event(): void
    {
        $eventDelegate = factory(EventDelegate::class)->create();
        $event = factory(Event::class)->create();

        $eventDelegate->signInTo($event);

        $this->assertTrue($eventDelegate->isSignedInAt($event));
    }

    /** @test */
    public function delegate_can_be_signed_out_of_an_event(): void
    {
        $eventDelegate = factory(EventDelegate::class)->create();
        $event = factory(Event::class)->create();

        $eventDelegate->signInTo($event);
        $this->assertTrue($eventDelegate->isSignedInAt($event));

        $eventDelegate->signOutFrom($event);
        $this->assertFalse($eventDelegate->isSignedInAt($event));
    }

    /** @test */
    public function delegate_can_toggle_signin_for_event(): void
    {
        $event = factory(Event::class)->create();
        $eventDelegate = factory(EventDelegate::class)->create();

        $eventDelegate->toggleSignInAt($event);
        $this->assertTrue($eventDelegate->isSignedInAt($event));
        $eventDelegate->toggleSignInAt($event);
        $this->assertFalse($eventDelegate->isSignedInAt($event));
    }

    /** @test */
    public function sign_in_and_out_more_than_once()
    {
        $event = factory(Event::class)->create();
        $eventDelegate = factory(EventDelegate::class)->create();

        $eventDelegate->toggleSignInAt($event);
        $this->assertTrue($eventDelegate->isSignedInAt($event));
        $eventDelegate->toggleSignInAt($event);
        $this->assertFalse($eventDelegate->isSignedInAt($event));
        $this->assertCount(1, SignIn::all());

        $eventDelegate->toggleSignInAt($event);
        $this->assertTrue($eventDelegate->isSignedInAt($event));
        $eventDelegate->toggleSignInAt($event);
        $this->assertFalse($eventDelegate->isSignedInAt($event));
        $this->assertCount(2, SignIn::all());
    }
}
